"{ Package: 'stx:libjava' }"

JavaRuntimeConstantPoolTests subclass:#JavaClassRefTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Java-Tests-RuntimeConstantPool'
!


!JavaClassRefTests methodsFor:'permissions tests'!

testAccessingNonPublicFromInside
    | javaClassRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/NonPublicClass;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'stx.libjava.tests.mocks.Crate').
    javaClassRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: ( throwedException isNil ).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 13:42:47 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingNonPublicFromOutside
    | javaClassRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/NonPublicClass;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaClassRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException notNil 
                and: [ throwedException messageText = 'IllegalAccessError' ]).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 13:37:41 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 13-04-2011 / 23:06:32 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingPublic
    | javaClassRef  initString |

    self enableMockedExceptionThrowing.
    self shouldnt: [
    initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaClassRef resolve.] raise: Error.
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 13:36:33 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testNonPublicClassPresent
    | javaClassRef  initString |

    self enableMockedExceptionThrowing.
    initString := 'Lstx/libjava/tests/mocks/NonPublicClass;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'stx.libjava.tests.mocks.SubclassOfNonPublicClass').
    javaClassRef resolve.
    self assertTrue: (javaClassRef valueCache notNil).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 13:38:49 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testPublicClassPresent
    | javaClassRef  initString |

    self enableMockedExceptionThrowing.
    initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaClassRef resolve.
    self assertTrue: (javaClassRef valueCache notNil).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 13:39:00 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaClassRefTests methodsFor:'resolving static tests'!

testCorrectStaticResolving
    | javaClassRef  initString  result  expectedResult |

    initString := 'Lstx/libjava/tests/mocks/ClassWithInnerClasses$InnerStatic;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'stx.libjava.tests.mocks.ClassWithInnerClasses').
    expectedResult := Java 
                classForName: 'stx.libjava.tests.mocks.ClassWithInnerClasses$InnerStatic'.
    result := javaClassRef resolveStatic.
    self assertTrue: (result = expectedResult).

    "Created: / 28-04-2011 / 21:51:26 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaClassRefTests methodsFor:'resolving tests'!

testCorrectInstanceCreation
| javaClassRef initString |

initString := 'Ljava/lang/String;'.
javaClassRef := JavaClassRef2 for: initString.
self assertFalse: (javaClassRef isResolved).
self assertTrue: (javaClassRef valueCache isNil).
self assertTrue: (javaClassRef name = initString).

    "Created: / 08-04-2011 / 14:01:41 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testCorrectResolving
    | javaClassRef  initString  result  expectedResult |

    initString := 'Ljava/lang/String;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    expectedResult := Java classForName: 'java.lang.String'.
    result := javaClassRef resolve.
    self assertTrue: (result = expectedResult).

    "Created: / 08-04-2011 / 14:07:57 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 11-04-2011 / 19:23:18 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidation
    | javaClassRef  initString |

    initString := 'Ljava/lang/String;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    self assertTrue: (javaClassRef isResolved not).
    javaClassRef resolve.
    self assertTrue: (javaClassRef isResolved).
    javaClassRef invalidate.
    self assertTrue: (javaClassRef isResolved not).

    "Created: / 08-04-2011 / 14:09:06 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 11-04-2011 / 19:41:20 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidationForClassNegative
    | javaClassRef  initString |

    initString := 'Ljava/lang/String;'.    
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    self assertTrue: (javaClassRef isResolved not).
    javaClassRef resolve.
    self assertTrue: (javaClassRef isResolved).
    javaClassRef invalidateForClass: 'Ljava/lang/Object;'.
    self assertTrue: (javaClassRef isResolved).

    "Created: / 08-04-2011 / 16:21:21 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 11-04-2011 / 19:41:40 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidationForClassPositive
    | javaClassRef  initString |

    initString := 'Ljava/lang/String;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    self assertTrue: (javaClassRef isResolved not).
    javaClassRef resolve.
    self assertTrue: (javaClassRef isResolved).
    javaClassRef invalidateForClass: initString.
    self assertTrue: (javaClassRef isResolved not).

    "Created: / 08-04-2011 / 16:21:35 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 11-04-2011 / 19:41:43 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testResolving
    | javaClassRef  initString |

    initString := 'Ljava/lang/String;'.
    javaClassRef := JavaClassRef2 for: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    self assertTrue: (javaClassRef isResolved not).
    javaClassRef resolve.
    self assertTrue: (javaClassRef isResolved).

    "Created: / 08-04-2011 / 14:04:01 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 11-04-2011 / 19:41:48 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaClassRefTests class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !

